<?php
/**
 * Created by PhpStorm.
 * User: jderosa
 * Date: 31/10/19
 * Time: 16.33
 */

namespace FiloBlu\Rma\Helper;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Psr\Log\LoggerInterface;
use FiloBlu\Rma\Block\Adminhtml\System\Config as Reason;
use FiloBlu\Rma\Helper\SerializerHelper;

class RmaHelper
{
    const CACHE_ID = 'rma_reasons';
    const RMA_REASON_CONFIG = 'sales/rma_reason/reason';
    const STANDARD_REASON_ID_FOR_CHANGESIZE = 10;

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var LoggerInterface 
     */
    protected $logger;

    /**
     * @var Reason
     */
    protected $reason;

    /**
     * @var \FiloBlu\Rma\Helper\SerializerHelper
     */
    protected $serializer;

    /**
     * RmaHelper constructor.
     * @param ScopeConfigInterface $scopeConfig
     * @param LoggerInterface $logger
     * @param Reason $reason
     * @param \FiloBlu\Rma\Helper\SerializerHelper $serializer
     */
    public function __construct(ScopeConfigInterface $scopeConfig,
                                LoggerInterface $logger,
                                Reason $reason,
                                SerializerHelper $serializer)
    {
        $this->scopeConfig = $scopeConfig;
        $this->logger = $logger;
        $this->reason = $reason;
        $this->serializer = $serializer;
    }



    /**
     * @param $reasonId
     * @return bool
     */
    public function checkIfItIsMapped($reasonId){
        $mapped = false;
        $value = $this->scopeConfig->getValue(self::RMA_REASON_CONFIG);
        if (empty($value)) return [];

        $mappedReasons = $this->serializer->unserialize($value);
        foreach ($mappedReasons as $reason){
            if($reasonId == (int)$reason['brand_reason']){
                $mapped = true;
            }
        }
        return $mapped;
    }


    /**
     * @return array
     */
    public function getAllMappedReason(){
        
        error_log(__METHOD__);

        $value = $this->scopeConfig->getValue(self::RMA_REASON_CONFIG);
        if (empty($value)) return [];

        $mappedReasons = $this->serializer->unserialize($value);
        $reasons = [];
        $reasons [] = ['value'=>"",'label'=>" "];
        $includedIds = [];
        foreach ($mappedReasons as $reason){
            $standardReasonId = $reason['standard_reason'];
            $standardReason = $this->getStandardReasonById($standardReasonId);
            if($standardReason && (int)$standardReason['visible_on_front'] == 1) {
                if(!in_array($standardReason['id'],$includedIds)){
                    $includedIds []=$standardReason['id'];
                    $reasons [] = ['value' => $standardReason['id'], 'label' => $standardReason['standard_name']];
                }

            }
        }
        return $reasons;
    }


    /**
     * @param int $reasonId
     * @param string $type
     * @return array
     */
    public function getMappedReasonByReasonId(int $reasonId,string $type='standard'){

        $value = $this->scopeConfig->getValue(self::RMA_REASON_CONFIG);
        if (empty($value)) return [];

        $mappedReasons = $this->serializer->unserialize($value);
        $exists = false;
        $result = [];
        foreach ($mappedReasons as $reason){
            if($type == 'standard'){
                if($reasonId == (int)$reason['standard_reason']){
                    $exists = true;
                    $result[] = (int)$reason['brand_reason'];
                }
            }else if($type=='brand'){
                if($reasonId == (int)$reason['brand_reason']){
                    $exists = true;
                    $result[] = (int)$reason['standard_reason'];
                }
            }
        }
        if($exists){
            return $result;
        }else{
            return [];
        }
    }

    /**
     * @param int $magentoReasonId
     * @return bool
     */
    public function checkIfAutoApproved(int $magentoReasonId)
    {
        $autoApprove = false;

        $value = $this->scopeConfig->getValue(self::RMA_REASON_CONFIG);
        if (empty($value)) return false;

        $mappedReasons = $this->serializer->unserialize($value);
        $standardReasons = $this->reason->getAll();
        foreach ($mappedReasons as $reason){
            if((int)$reason['brand_reason'] == $magentoReasonId){
                foreach ($standardReasons as $stdReason){
                    if($stdReason['id'] == $reason['standard_reason'] && $stdReason['automatic_approval']){
                        $autoApprove =true;
                    }
                }
            }
        }

        return $autoApprove;
    }

    /**
     * @param int $reasonId
     * @return bool
     */
    public function checkIfAllowedMagentoChangeSizeReason(int $reasonId){
        if($this->checkIfItIsMapped($reasonId)){
             if(in_array(self::STANDARD_REASON_ID_FOR_CHANGESIZE,$this->getMappedReasonByReasonId($reasonId,'brand'))){
                 return true;
             }
             else{
                 return false;
            }
        }else{
            $this->logger->critical("RmaHelper -> Standard Change Size Rma Reason isn't mapped yet!");
            return false;
        }
        
    }
    
    
}